var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

import React, { Component } from 'react';
import { Provider } from 'react-redux';
import { createStore, applyMiddleware, combineReducers, compose } from 'redux';
import thunkMiddleware from 'redux-thunk';
import { createLogger } from 'redux-logger';
import { persistStore, autoRehydrate } from 'redux-persist';

import { applicationShape } from './with';

var _controllers = Symbol('controllers');
var _store = Symbol('store');
var _options = Symbol('options');
var _controllerName = Symbol('controllerName');

var CONTROLLER_SUFFIX = 'Controller';

var Application = function () {
    function Application(options) {
        _classCallCheck(this, Application);

        this[_controllers] = {};
        this[_options] = Object.assign({
            noPersistance: false
        }, options);
    }

    Application.prototype.register = function register(controller) {
        var cls = void 0,
            name = void 0;

        if (typeof controller === 'function') {
            cls = controller;
            name = controller.controllerName;

            if (!name) {
                name = cls.name;
                if (name.endsWith(CONTROLLER_SUFFIX) && name.length > CONTROLLER_SUFFIX.length) {
                    name = name.substring(0, name.length - CONTROLLER_SUFFIX.length);
                }
            }
        } else if (controller && controller.cls) {
            cls = controller.cls;
            name = controller.name;
        } else {
            throw new Error('Invalid controller specified');
        }

        for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
            args[_key - 1] = arguments[_key];
        }

        var instance = new (Function.prototype.bind.apply(cls, [null].concat([this, [name]].concat(args))))();
        if (this[_controllers][name]) {
            console.log('WARNING: Controller for name "' + name + '" was already registered, overwriting');
        }
        instance[_controllerName] = name;

        this[_controllers][name] = instance;
    };

    Application.prototype.getController = function getController(name) {
        var controller = this[_controllers][name];
        if (!controller && !name.endsWith(CONTROLLER_SUFFIX)) {
            controller = this[_controllers][name + CONTROLLER_SUFFIX];
        }
        return controller;
    };

    Application.prototype.getControllerName = function getControllerName(controller) {
        return controller[_controllerName];
    };

    Application.prototype.run = function run(starterComponent, splashScreenComponent) {
        var _this = this;

        var usePersistStore = persistStore && !this[_options].noPersistance;

        var rootComponent = void 0;

        var reducers = {};
        var hasReducers = false;
        for (var controllerKey in this[_controllers]) {
            // eslint-disable-line guard-for-in
            var controller = this[_controllers][controllerKey];
            var reducer = controller.reducer();
            if (reducer) {
                reducers[controllerKey] = reducer;
                hasReducers = true;
            }
        }

        var loggerMiddleware = __DEV__ && createLogger ? createLogger() : undefined;
        this[_store] = createStore(hasReducers ? combineReducers(reducers) : function () {
            return {};
        }, {}, compose.apply(undefined, [usePersistStore ? autoRehydrate({
            log: true
        }) : undefined, applyMiddleware.apply(undefined, [thunkMiddleware, loggerMiddleware].filter(function (func) {
            return func !== undefined;
        }))].filter(function (func) {
            return func !== undefined;
        })));

        var isStoreLoaded = !usePersistStore;
        function setIsStoreLoaded(loaded) {
            isStoreLoaded = loaded;
            if (rootComponent) {
                rootComponent.setState({
                    isStoreLoaded: isStoreLoaded
                });
            }
        }

        if (usePersistStore) {
            persistStore(this[_store], { storage: this[_options].persistStorage }, function () {
                console.log('Store rehydration complete');

                for (var _controllerKey in _this[_controllers]) {
                    // eslint-disable-line guard-for-in
                    var _controller = _this[_controllers][_controllerKey];
                    _controller.afterRehydrate();
                }

                setIsStoreLoaded(true);
            });
        }

        for (var _controllerKey2 in this[_controllers]) {
            // eslint-disable-line guard-for-in
            var _controller2 = this[_controllers][_controllerKey2];
            _controller2.beforeRun();
        }

        var application = this;

        var ApplicationRoot = function (_Component) {
            _inherits(ApplicationRoot, _Component);

            function ApplicationRoot(props) {
                _classCallCheck(this, ApplicationRoot);

                var _this2 = _possibleConstructorReturn(this, _Component.call(this, props));

                _this2.state = {
                    isStoreLoaded: isStoreLoaded
                };
                return _this2;
            }

            ApplicationRoot.prototype.getChildContext = function getChildContext() {
                return {
                    application: application
                };
            };

            ApplicationRoot.prototype.render = function render() {
                return React.createElement(
                    Provider,
                    { store: this.props.store },
                    this.state.isStoreLoaded ? starterComponent : splashScreenComponent
                );
            };

            return ApplicationRoot;
        }(Component);

        ApplicationRoot.childContextTypes = {
            application: applicationShape
        };

        return React.createElement(ApplicationRoot, {
            ref: function ref(_ref) {
                return rootComponent = _ref;
            },
            store: this.store
        });
    };

    _createClass(Application, [{
        key: 'store',
        get: function get() {
            return this[_store];
        }
    }]);

    return Application;
}();

export { Application as default };