function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

import { connect } from "react-redux";
import { withController } from "./with";

var Mapper = function () {
    function Mapper(mappings) {
        _classCallCheck(this, Mapper);

        this.mappings = mappings.map(function (mapping) {
            if (typeof mapping === "string") {
                return [{
                    controllerName: mapping,
                    prop: undefined,
                    mapStateToProps: function mapStateToProps(controller, state, ownProps) {
                        return controller.mapStateToProps(state, ownProps);
                    },
                    mapDispatchToProps: function mapDispatchToProps(controller, dispatch, ownProps) {
                        return controller.mapDispatchToProps(dispatch, ownProps);
                    }
                }];
            } else {
                var keys = Object.keys(mapping);
                return keys.map(function (controllerName) {
                    mapping = mapping[controllerName];

                    var overrideProps = {};
                    for (var propKey in mapping) {
                        if (!/^\$/.test(propKey)) {
                            overrideProps[propKey] = mapping[propKey];
                        }
                    }

                    return {
                        controllerName: controllerName,
                        prop: mapping.$prop,
                        mapStateToProps: mapping.$mapStateToProps ? mapping.$mapStateToProps : function (controller, state, ownProps) {
                            return controller.mapStateToProps(state, ownProps, {
                                pickProps: mapping.$pickProps,
                                omitProps: mapping.$omitProps,
                                overrideProps: overrideProps
                            });
                        },
                        mapDispatchToProps: mappings.$mapDispatchToProps ? mapping.$mapDispatchToProps : function (controller, dispatch, ownProps) {
                            return controller.mapDispatchToProps(dispatch, ownProps, {
                                pickProps: mapping.$pickProps,
                                omitProps: mapping.$omitProps,
                                overrideProps: overrideProps
                            });
                        }
                    };
                });
            }
        }).reduce(function (acc, cur) {
            return acc.concat(cur);
        }, []); // flatten array
    }

    Mapper.prototype.controllersNames = function controllersNames() {
        return this.mappings.map(function (mapping) {
            return mapping.controllerName;
        });
    };

    Mapper.prototype.mapStateToProps = function mapStateToProps(state, ownProps) {
        return this.mappings.reduce(function (result, mapping) {
            var props = mapping.mapStateToProps(ownProps.controllers[mapping.controllerName], state, ownProps);

            if (mapping.prop) {
                var _Object$assign;

                Object.assign(result, (_Object$assign = {}, _Object$assign[mapping.prop] = props, _Object$assign));
            } else {
                Object.assign(result, props);
            }

            return result;
        }, {});
    };

    Mapper.prototype.mapDispatchToProps = function mapDispatchToProps(dispatch, ownProps) {
        return this.mappings.reduce(function (result, mapping) {
            var props = mapping.mapDispatchToProps(ownProps.controllers[mapping.controllerName], dispatch, ownProps);

            if (mapping.prop) {
                var _Object$assign2;

                Object.assign(result, (_Object$assign2 = {}, _Object$assign2[mapping.prop] = props, _Object$assign2));
            } else {
                Object.assign(result, props);
            }

            return result;
        }, {});
    };

    return Mapper;
}();

export default function Container(WrappedComponent) {
    for (var _len = arguments.length, mappings = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        mappings[_key - 1] = arguments[_key];
    }

    var mapper = new Mapper(mappings);

    return withController.apply(undefined, [connect(mapper.mapStateToProps.bind(mapper), mapper.mapDispatchToProps.bind(mapper))(WrappedComponent)].concat(mapper.controllersNames()));
}