import React from 'react';
import createReactClass from 'create-react-class';
import PropTypes from 'prop-types';
import hoistStatics from 'hoist-non-react-statics';

var shape = PropTypes.shape,
    func = PropTypes.func,
    object = PropTypes.object;

export var applicationShape = shape({
    store: object.isRequired,
    getController: func.isRequired
});

function getDisplayName(WrappedComponent) {
    return WrappedComponent.displayName || WrappedComponent.name || 'Component';
}

export function withApplication(WrappedComponent) {
    var WithApplication = createReactClass({
        displayName: 'WithApplication',

        contextTypes: { application: applicationShape },
        propTypes: { application: applicationShape },

        render: function render() {
            var _this = this;

            var application = this.props.application || this.context.application;
            var props = Object.assign({}, this.props, { application: application });

            return React.createElement(WrappedComponent, props);
        }
    });

    WithApplication.displayName = 'withApplication(' + getDisplayName(WrappedComponent) + ')';
    WithApplication.WrappedComponent = WrappedComponent;

    return hoistStatics(WithApplication, WrappedComponent);
}

export function withController(WrappedComponent, controllerName) {
    for (var _len = arguments.length, otherControllerNames = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
        otherControllerNames[_key - 2] = arguments[_key];
    }

    var WithController = createReactClass({
        displayName: 'WithController',

        contextTypes: { application: applicationShape },
        propTypes: { application: applicationShape },

        render: function render() {
            var _this = this;

            var application = this.props.application || this.context.application;
            var controller = application ? application.getController(controllerName) : undefined;
            var controllers;

            if (application) {
                controllers = {};
                controllers[controllerName] = controller;

                if (!controller) console.warn('WARNING: Controller "' + controllerName + '" is not registered');

                if (otherControllerNames) {
                    for (var _iterator = otherControllerNames, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
                        var _ref;

                        if (_isArray) {
                            if (_i >= _iterator.length) break;
                            _ref = _iterator[_i++];
                        } else {
                            _i = _iterator.next();
                            if (_i.done) break;
                            _ref = _i.value;
                        }

                        var otherControllerName = _ref;

                        controllers[otherControllerName] = application.getController(otherControllerName);

                        if (!controllers[otherControllerName]) {
                            console.warn('WARNING: Controller "' + otherControllerName + '" is not registered');
                        }
                    }
                }
            }

            var props = Object.assign({}, this.props, {
                controller: controller,
                controllers: controllers
            });

            return React.createElement(WrappedComponent, props);
        }
    });

    WithController.displayName = 'withController(' + getDisplayName(WrappedComponent) + ')';
    WithController.WrappedComponent = WrappedComponent;

    return hoistStatics(WithController, WrappedComponent);
}