'use strict';

exports.__esModule = true;
exports.default = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _Action = require('./utils/Action');

var _Action2 = _interopRequireDefault(_Action);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var _application = Symbol('application');
var _mountPath = Symbol('mountPath');
var _mountPathString = Symbol('mountPathString');
var _actions = Symbol('actions');

var getAllClassMethods = function getAllClassMethods(obj) {
    var keys = [],
        topObject = obj;

    do {
        var l = Object.getOwnPropertyNames(obj).sort().filter(function (p, i, arr) {
            return typeof topObject[p] === 'function' && //only the methods
            p !== 'constructor' && ( //not the constructor
            i == 0 || p !== arr[i - 1]) && //not overriding in this prototype
            keys.indexOf(p) === -1;
        } //not overridden in a child
        );
        keys = keys.concat(l);
    } while ((obj = Object.getPrototypeOf(obj)) && //walk-up the prototype chain
    Object.getPrototypeOf(obj) //not the the Object prototype methods (hasOwnProperty, etc...)
    );

    return keys;
};

var Controller = function () {
    function Controller(application, mountPath) {
        _classCallCheck(this, Controller);

        this[_application] = application;
        this[_mountPath] = mountPath;
        this[_mountPathString] = mountPath.join('.');
        this[_actions] = {};
    }

    Controller.prototype.createAction = function createAction(action, key) {
        if (typeof action === "string") {
            this[_actions][action] = new _Action2.default(this[_mountPath] + "." + action);
        } else {
            this[_actions][key || action.type()] = action;
        }
    };

    Controller.prototype.dispatchAction = function dispatchAction(actionType, payload) {
        var dotI = actionType.indexOf(".");
        var actionBaseType = dotI === -1 ? actionType : actionType.substring(0, dotI);
        var actionStage = dotI === -1 ? undefined : actionType.substring(dotI + 1);

        this.application.store.dispatch(this.actions[actionBaseType].action(actionStage, payload));
    };

    Controller.prototype.rootState = function rootState(state) {
        var innerState = state;
        this[_mountPath].forEach(function (key) {
            innerState = innerState[key];
        });
        return innerState;
    };

    Controller.prototype.$ = function $(state) {
        var innerState = state;
        this[_mountPath].forEach(function (key) {
            innerState = innerState[key];
        });
        return innerState;
    };

    Controller.prototype.reducer = function reducer() {
        // to be overriden in children
    };

    Controller.prototype.beforeRun = function beforeRun() {
        // to be overriden in children
    };

    Controller.prototype.afterRehydrate = function afterRehydrate() {
        // to be overriden in children
    };

    Controller.prototype.mapStateToProps = function mapStateToProps(state, context) {
        var _this = this;

        var _ref = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
            pickProps = _ref.pickProps,
            omitProps = _ref.omitProps,
            overrideProps = _ref.overrideProps;

        var props = {};

        if (!overrideProps) {
            overrideProps = {};
        }

        getAllClassMethods(this).forEach(function (key) {
            if (!((!(pickProps && pickProps.indexOf(propName) === -1) || !(omitProps && omitProps.indexOf(propName) !== -1)) && !_this[key].doNotMap && !_this[key].mapAsDispatch)) {
                return;
            }

            var shouldMap = false,
                propName;
            if (_this[key].mapAsStateFunction) {
                shouldMap = true;
                propName = key;
            } else if (/^get/.test(key)) {
                shouldMap = true;
                propName = key.substring(3, 4).toLowerCase() + key.substring(4);
            } else if (/^is/.test(key)) {
                shouldMap = true;
                propName = key;
            }

            if (shouldMap) {
                if (overrideProps[propName]) {
                    if (typeof overrideProps[propName] === "function") {
                        props[propName] = overrideProps[propName](_this, state, context, { pickProps: pickProps, omitProps: omitProps, overrideProps: overrideProps });
                    } else {
                        props[propName] = overrideProps[propName].$func ? overrideProps[propName].$func : overrideProps[propName];
                    }
                } else {
                    if (_this[key].mapAsStateFunction) {
                        props[propName] = function () {
                            for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
                                args[_key] = arguments[_key];
                            }

                            return _this[key].apply(_this, args.concat([state, context]));
                        };
                    } else {
                        props[propName] = _this[key](state, context);
                    }
                }
            }
        });

        return props;
    };

    Controller.prototype.mapDispatchToProps = function mapDispatchToProps(dispatch, context) {
        var _this2 = this;

        var _ref2 = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
            pickProps = _ref2.pickProps,
            omitProps = _ref2.omitProps,
            overrideProps = _ref2.overrideProps;

        var props = {};
        var defaultOmitFunctions = ["rootState", "$", "reducer", "beforeRun", "afterRehydrate", "mapStateToProps", "mapDispatchToProps", "doNotMap", "mapAsStateFunction", "mapAsDispatch"];

        if (!overrideProps) {
            overrideProps = {};
        }

        getAllClassMethods(this).forEach(function (key) {
            if ((!/^(get|_)/.test(key) && !/^(is|_)/.test(key) || _this2[key].mapAsDispatch) && defaultOmitFunctions.indexOf(key) === -1 && !_this2[key].doNotMap && !_this2[key].mapAsStateFunction) {
                if (overrideProps[key]) {
                    if (typeof overrideProps[key] === "function") {
                        props[key] = overrideProps[key](_this2, dispatch, context, { pickProps: pickProps, omitProps: omitProps, overrideProps: overrideProps });
                    } else {
                        props[key] = overrideProps[key].$func ? overrideProps[key].$func : overrideProps[key];
                    }
                } else {
                    props[key] = function () {
                        for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
                            args[_key2] = arguments[_key2];
                        }

                        return _this2[key].apply(_this2, args.concat([dispatch, context]));
                    };
                }
            }
        });

        return props;
    };

    Controller.prototype.doNotMap = function doNotMap() {
        for (var _len3 = arguments.length, functions = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
            functions[_key3] = arguments[_key3];
        }

        functions.forEach(function (func) {
            func.doNotMap = true;
        });
    };

    Controller.prototype.mapAsStateFunction = function mapAsStateFunction() {
        for (var _len4 = arguments.length, functions = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
            functions[_key4] = arguments[_key4];
        }

        functions.forEach(function (func) {
            func.mapAsStateFunction = true;
        });
    };

    Controller.prototype.mapAsDispatch = function mapAsDispatch() {
        for (var _len5 = arguments.length, functions = Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
            functions[_key5] = arguments[_key5];
        }

        functions.forEach(function (func) {
            func.mapAsDispatch = true;
        });
    };

    Controller.named = function named(name) {
        return {
            cls: this,
            name: name
        };
    };

    _createClass(Controller, [{
        key: 'application',
        get: function get() {
            return this[_application];
        }
    }, {
        key: 'mountPath',
        get: function get() {
            return this[_mountPath];
        }
    }, {
        key: 'mountPathString',
        get: function get() {
            return this[_mountPathString];
        }
    }, {
        key: 'actions',
        get: function get() {
            return this[_actions];
        }
    }]);

    return Controller;
}();

exports.default = Controller;