'use strict';

exports.__esModule = true;
exports.applicationShape = undefined;
exports.withApplication = withApplication;
exports.withController = withController;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _createReactClass = require('create-react-class');

var _createReactClass2 = _interopRequireDefault(_createReactClass);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _hoistNonReactStatics = require('hoist-non-react-statics');

var _hoistNonReactStatics2 = _interopRequireDefault(_hoistNonReactStatics);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var shape = _propTypes2.default.shape,
    func = _propTypes2.default.func,
    object = _propTypes2.default.object;
var applicationShape = exports.applicationShape = shape({
    store: object.isRequired,
    getController: func.isRequired
});

function getDisplayName(WrappedComponent) {
    return WrappedComponent.displayName || WrappedComponent.name || 'Component';
}

function withApplication(WrappedComponent) {
    var WithApplication = (0, _createReactClass2.default)({
        displayName: 'WithApplication',

        contextTypes: { application: applicationShape },
        propTypes: { application: applicationShape },

        render: function render() {
            var _this = this;

            var application = this.props.application || this.context.application;
            var props = Object.assign({}, this.props, { application: application });

            return _react2.default.createElement(WrappedComponent, props);
        }
    });

    WithApplication.displayName = 'withApplication(' + getDisplayName(WrappedComponent) + ')';
    WithApplication.WrappedComponent = WrappedComponent;

    return (0, _hoistNonReactStatics2.default)(WithApplication, WrappedComponent);
}

function withController(WrappedComponent, controllerName) {
    for (var _len = arguments.length, otherControllerNames = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
        otherControllerNames[_key - 2] = arguments[_key];
    }

    var WithController = (0, _createReactClass2.default)({
        displayName: 'WithController',

        contextTypes: { application: applicationShape },
        propTypes: { application: applicationShape },

        render: function render() {
            var _this = this;

            var application = this.props.application || this.context.application;
            var controller = application ? application.getController(controllerName) : undefined;
            var controllers;

            if (application) {
                controllers = {};
                controllers[controllerName] = controller;

                if (!controller) console.warn('WARNING: Controller "' + controllerName + '" is not registered');

                if (otherControllerNames) {
                    for (var _iterator = otherControllerNames, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
                        var _ref;

                        if (_isArray) {
                            if (_i >= _iterator.length) break;
                            _ref = _iterator[_i++];
                        } else {
                            _i = _iterator.next();
                            if (_i.done) break;
                            _ref = _i.value;
                        }

                        var otherControllerName = _ref;

                        controllers[otherControllerName] = application.getController(otherControllerName);

                        if (!controllers[otherControllerName]) {
                            console.warn('WARNING: Controller "' + otherControllerName + '" is not registered');
                        }
                    }
                }
            }

            var props = Object.assign({}, this.props, {
                controller: controller,
                controllers: controllers
            });

            return _react2.default.createElement(WrappedComponent, props);
        }
    });

    WithController.displayName = 'withController(' + getDisplayName(WrappedComponent) + ')';
    WithController.WrappedComponent = WrappedComponent;

    return (0, _hoistNonReactStatics2.default)(WithController, WrappedComponent);
}